@@ -39,6 +39,7 @@ hypre_ParVectorCreate( MPI_Comm      comm,
       return NULL;
    }
    vector = hypre_CTAlloc(hypre_ParVector, 1, HYPRE_MEMORY_HOST);
+
    hypre_MPI_Comm_rank(comm, &my_id);
 
    if (!partitioning_in)
@@ -112,22 +113,53 @@ hypre_ParVectorDestroy( hypre_ParVector *vector )
 }
 
 /*--------------------------------------------------------------------------
- * hypre_ParVectorInitialize_v2
- *
- * Initialize a hypre_ParVector at a given memory location
+ * hypre_ParVectorInitializeShell
  *--------------------------------------------------------------------------*/
 
 HYPRE_Int
-hypre_ParVectorInitialize_v2( hypre_ParVector *vector, HYPRE_MemoryLocation memory_location )
+hypre_ParVectorInitializeShell(hypre_ParVector *vector)
 {
    if (!vector)
    {
       hypre_error_in_arg(1);
       return hypre_error_flag;
    }
-   hypre_SeqVectorInitialize_v2(hypre_ParVectorLocalVector(vector), memory_location);
 
-   hypre_ParVectorActualLocalSize(vector) = hypre_VectorSize(hypre_ParVectorLocalVector(vector));
+   hypre_Vector *local_vector = hypre_ParVectorLocalVector(vector);
+
+   hypre_SeqVectorInitializeShell(local_vector);
+   hypre_ParVectorActualLocalSize(vector) = hypre_VectorSize(local_vector);
+
+   return hypre_error_flag;
+}
+
+/*--------------------------------------------------------------------------
+ * hypre_ParVectorSetData
+ *--------------------------------------------------------------------------*/
+
+HYPRE_Int
+hypre_ParVectorSetData(hypre_ParVector *vector,
+                       HYPRE_Complex   *data)
+{
+   hypre_SeqVectorSetData(hypre_ParVectorLocalVector(vector), data);
+
+   return hypre_error_flag;
+}
+
+/*--------------------------------------------------------------------------
+ * hypre_ParVectorInitialize_v2
+ *
+ * Initialize a hypre_ParVector at a given memory location
+ *--------------------------------------------------------------------------*/
+
+HYPRE_Int
+hypre_ParVectorInitialize_v2( hypre_ParVector      *vector,
+                              HYPRE_MemoryLocation  memory_location )
+{
+   hypre_Vector *local_vector = hypre_ParVectorLocalVector(vector);
+
+   hypre_ParVectorInitializeShell(vector);
+   hypre_SeqVectorInitialize_v2(local_vector, memory_location);
 
    return hypre_error_flag;
 }
@@ -372,6 +404,22 @@ hypre_ParVectorCopy( hypre_ParVector *x,
    return hypre_SeqVectorCopy(x_local, y_local);
 }
 
+/*--------------------------------------------------------------------------
+ * hypre_ParVectorStridedCopy
+ *--------------------------------------------------------------------------*/
+
+HYPRE_Int
+hypre_ParVectorStridedCopy( hypre_ParVector *x,
+                            HYPRE_Int        istride,
+                            HYPRE_Int        ostride,
+                            HYPRE_Int        size,
+                            HYPRE_Complex   *data)
+{
+   hypre_Vector *x_local = hypre_ParVectorLocalVector(x);
+
+   return hypre_SeqVectorStridedCopy(x_local, istride, ostride, size, data);
+}
+
 /*--------------------------------------------------------------------------
  * hypre_ParVectorCloneShallow
  *
@@ -576,7 +624,7 @@ hypre_VectorToParVector ( MPI_Comm      comm,
    HYPRE_Int           global_vecstride, vecstride, idxstride;
    hypre_ParVector    *par_vector;
    hypre_Vector       *local_vector;
-   HYPRE_Complex      *v_data;
+   HYPRE_Complex      *v_data = NULL;
    HYPRE_Complex      *local_data;
    hypre_MPI_Request  *requests;
    hypre_MPI_Status   *status, status0;
@@ -958,51 +1006,72 @@ hypre_ParVectorPrintIJ( hypre_ParVector *vector,
                         const char      *filename )
 {
    MPI_Comm          comm;
-   HYPRE_BigInt      global_size, j;
    HYPRE_BigInt     *partitioning;
-   HYPRE_Complex    *local_data;
-   HYPRE_Int         myid, num_procs, i, part0;
-   char              new_filename[255];
+   hypre_Vector     *local_vector;
+   HYPRE_Int         local_size;
+   HYPRE_Int         myid, num_procs, i, j;
+   char              new_filename[HYPRE_MAX_FILE_NAME_LEN];
+   char              msg[1024];
    FILE             *file;
+
    if (!vector)
    {
       hypre_error_in_arg(1);
       return hypre_error_flag;
    }
    comm         = hypre_ParVectorComm(vector);
-   global_size  = hypre_ParVectorGlobalSize(vector);
    partitioning = hypre_ParVectorPartitioning(vector);
-
-   /* multivector code not written yet */
-   hypre_assert( hypre_ParVectorNumVectors(vector) == 1 );
-   if ( hypre_ParVectorNumVectors(vector) != 1 ) { hypre_error_in_arg(1); }
+   local_vector = hypre_ParVectorLocalVector(vector);
+   local_size   = hypre_VectorSize(local_vector);
 
    hypre_MPI_Comm_rank(comm, &myid);
    hypre_MPI_Comm_size(comm, &num_procs);
 
    hypre_sprintf(new_filename, "%s.%05d", filename, myid);
-
    if ((file = fopen(new_filename, "w")) == NULL)
    {
-      hypre_error_w_msg(HYPRE_ERROR_GENERIC, "Error: can't open output file %s\n");
+      hypre_sprintf(msg, "Error: cannot open output file: %s", new_filename);
+      hypre_error_w_msg(HYPRE_ERROR_GENERIC, msg);
       return hypre_error_flag;
    }
 
-   local_data = hypre_VectorData(hypre_ParVectorLocalVector(vector));
+   /* Write header: global partitioning */
+   hypre_fprintf(file, "%b %b\n", partitioning[0] + base_j, partitioning[1] + base_j - 1);
 
-   hypre_fprintf(file, "%b \n", global_size);
-   for (i = 0; i < 2; i++)
+   /* Write additional header line in the case of multi-component vectors */
+   if (hypre_ParVectorNumVectors(vector) > 1)
    {
-      hypre_fprintf(file, "%b ", partitioning[i] + base_j);
-   }
-   hypre_fprintf(file, "\n");
-
-   part0 = partitioning[0];
-   for (j = part0; j < partitioning[1]; j++)
-   {
-      hypre_fprintf(file, "%b %.14e\n", j + base_j, local_data[(HYPRE_Int)(j - part0)]);
+      hypre_fprintf(file, "%d %d %d %d\n",
+                    hypre_VectorNumVectors(local_vector),
+                    hypre_VectorMultiVecStorageMethod(local_vector),
+                    hypre_VectorVectorStride(local_vector),
+                    hypre_VectorIndexStride(local_vector));
    }
 
+   /* Write coefficients */
+   if (hypre_ParVectorNumVectors(vector) > 1)
+   {
+      /* Multi-component vectors */
+      for (i = 0; i < local_size; i++)
+      {
+         hypre_fprintf(file, "%b", (HYPRE_BigInt) (i + base_j) + partitioning[0]);
+         for (j = 0; j < hypre_VectorNumVectors(local_vector); j++)
+         {
+            hypre_fprintf(file, " %.14e", hypre_VectorEntryIJ(local_vector, i, j));
+         }
+         hypre_fprintf(file, "\n");
+      }
+   }
+   else
+   {
+      /* Single-component (regular) vectors */
+      for (j = 0; j < local_size; j++)
+      {
+         hypre_fprintf(file, "%b %.14e\n",
+                       (HYPRE_BigInt) (j + base_j) + partitioning[0],
+                       hypre_VectorEntryI(local_vector, j));
+      }
+   }
    fclose(file);
 
    return hypre_error_flag;
@@ -1041,13 +1110,12 @@ hypre_ParVectorPrintBinaryIJ( hypre_ParVector *par_vector,
    hypre_Vector          *h_vector;
    HYPRE_Int              size;
    HYPRE_Int              num_components;
-   HYPRE_Int              total_size;
    HYPRE_Int              storage_method;
 
    /* Local variables */
    char                   new_filename[HYPRE_MAX_FILE_NAME_LEN];
    FILE                  *fp;
-   size_t                 count;
+   size_t                 count, total_size;
    hypre_uint64           header[8];
    HYPRE_Int              one = 1;
    HYPRE_Complex         *data;
@@ -1142,11 +1210,12 @@ hypre_ParVectorReadIJ( MPI_Comm          comm,
    hypre_ParVector  *vector;
    hypre_Vector     *local_vector;
    HYPRE_Complex    *local_data;
-   HYPRE_BigInt      partitioning[2];
+   HYPRE_BigInt      big_local_size;
+   HYPRE_BigInt      partitioning[2] = {0, 0};
    HYPRE_Int         base_j;
 
-   HYPRE_Int         myid, num_procs, i, j;
-   char              new_filename[255];
+   HYPRE_Int         myid, num_procs, j;
+   char              new_filename[HYPRE_MAX_FILE_NAME_LEN];
    FILE             *file;
 
    hypre_MPI_Comm_size(comm, &num_procs);
@@ -1160,24 +1229,21 @@ hypre_ParVectorReadIJ( MPI_Comm          comm,
       return hypre_error_flag;
    }
 
-   hypre_fscanf(file, "%b", &global_size);
    /* this may need to be changed so that the base is available in the file! */
-   hypre_fscanf(file, "%b", partitioning);
-   for (i = 0; i < 2; i++)
-   {
-      hypre_fscanf(file, "%b", partitioning + i);
-   }
+   hypre_fscanf(file, "%b %b", partitioning[0], partitioning[1]);
+   big_local_size = partitioning[1] - partitioning[0] + 1;
+   hypre_MPI_Allreduce(&big_local_size, &global_size, 1, HYPRE_MPI_BIG_INT,
+                       hypre_MPI_SUM, comm);
+
    /* This is not yet implemented correctly! */
    base_j = 0;
-   vector = hypre_ParVectorCreate(comm, global_size,
-                                  partitioning);
-
-   hypre_ParVectorInitialize(vector);
+   vector = hypre_ParVectorCreate(comm, global_size, partitioning);
+   hypre_ParVectorInitialize_v2(vector, HYPRE_MEMORY_HOST);
 
    local_vector = hypre_ParVectorLocalVector(vector);
    local_data   = hypre_VectorData(local_vector);
 
-   for (j = 0; j < (HYPRE_Int)(partitioning[1] - partitioning[0]); j++)
+   for (j = 0; j < (HYPRE_Int) big_local_size; j++)
    {
       hypre_fscanf(file, "%b %le", &J, local_data + j);
    }
@@ -1209,6 +1275,8 @@ hypre_FillResponseParToVectorAll( void       *p_recv_contact_buf,
                                   void      **p_send_response_buf,
                                   HYPRE_Int  *response_message_size )
 {
+   HYPRE_UNUSED_VAR(p_send_response_buf);
+
    HYPRE_Int     myid;
    HYPRE_Int     i, index, count, elength;
 
